package ru.dbotthepony.kommons.io

import ru.dbotthepony.kommons.util.KOptional
import java.io.Closeable
import java.io.File
import java.util.concurrent.CompletableFuture

/**
 * For actual implementation, see [BTreeDB6].
 *
 *
 */
abstract class BTreeDB<K, V> : Closeable {
	abstract val file: File
	abstract val blockSize: Int

	open operator fun contains(key: K): Boolean {
		return hasKey(key).get()
	}

	abstract fun hasKey(key: K): CompletableFuture<Boolean>

	/**
	 * Reads data at specified [key]
	 */
	abstract fun read(key: K): CompletableFuture<KOptional<V>>
	abstract fun findAllKeys(): CompletableFuture<List<K>>

	abstract fun write(key: K, value: V): CompletableFuture<*>
}

abstract class ByteDataBTreeDB<K> : BTreeDB<K, ByteArray>() {
	abstract fun write(key: K, value: ByteArray, offset: Int, length: Int): CompletableFuture<*>

	final override fun write(key: K, value: ByteArray): CompletableFuture<*> {
		return write(key, value, 0, value.size)
	}
}
